function setPackInfo(packInfo) {
    document.getElementById('title').value = packInfo.name || ''
    document.getElementById('description').value = packInfo.description || ''
    document.getElementById('short-description').value = packInfo.shortDescription || ''
    document.getElementById('author').value = packInfo.author || ''
    document.getElementById('author-link').value = packInfo.authorLink || ''
    document.getElementById('free-toy').value = packInfo.freeToy || ''
    document.getElementById('tags').value = packInfo.tags ? packInfo.tags.join(',') : ''
    document.getElementById('color-picker').value = packInfo.colour || '#ff0000'
    document.getElementById('color-picker').style.backgroundColor = packInfo.colour || '#ff0000'
    const imagePreview = document.getElementById('image-preview')
    if (packInfo.image) {
        imagePreview.src = packInfo.image
        imagePreview.style.display = 'block'
    } else {
        imagePreview.src = ''
        imagePreview.style.display = 'none'
    }

    const audioPreview = document.getElementById('audio-preview')
    if (packInfo.audioPreview) {
        audioPreview.src = packInfo.audioPreview
        audioPreview.style.display = 'block'
        audioPreview.load()
    } else {
        audioPreview.style.display = 'none'
        audioPreview.src = ''
    }
}

function updatePackInfo() {
    if (editingPack === '') {
        console.error(`Editing pack is null`)
        return
    }

    const form = document.getElementById('pack-details-form')
    const formData = {
        title: form.title.value,
        description: form.description.value,
        shortDescription: document.getElementById('short-description').value,
        image: document.getElementById('image-preview').src,
        audioPreview: document.getElementById('audio-preview').src,
        tags: form.tags.value.split(/[ ,;]+/).filter(Boolean),
        author: form.author.value,
        authorLink: form['author-link'].value,
        previewLink: form['preview-link'].value,
        freeToy: form['free-toy'].value,
        colour: document.getElementById('color-picker').value,
    }
    console.log(formData)
    sendUpdatePack(formData)
}

function listAllPacks(directories) {
    const tableBody = document.getElementById('pack-table-list')
    tableBody.innerHTML = ''

    directories.samplePacks.forEach((packName) => {
        const row = document.createElement('tr')
        row.dataset.id = packName
        row.appendChild(makeBooleanCell('false'))
        row.appendChild(makeLabelCell(packName))
        row.appendChild(makeLabelCell('ERRORS'))
        row.addEventListener('click', () => {
            const clickedSelf = editingPackRow === row
            if (editingPackRow) {
                editingPackRow.classList.remove('active')
            }

            if (!clickedSelf) {
                editingPack = packName
                editingPackRow = row
                row.classList.add('active')
            }

            togglePackEditor(row, packName)
        })
        tableBody.appendChild(row)
        requestPackStatus(packName)
    })
}

function setSamplePacks(directoriesJson) {
    document.querySelectorAll('.pack-title').forEach((title) => title.classList.remove('active'))

    hidePackEditor()
    listAllPacks(JSON.parse(directoriesJson))
}
