function assignPads(indices, categories, pads, usedSamples, categorizedSamples) {
	indices.forEach((index) => {
		for (const category of categories) {
			if (categorizedSamples[category] && categorizedSamples[category].length > 0) {
				const sample = categorizedSamples[category].shift()
				pads[index] = sample
				usedSamples.add(sample)
				break
			}
		}
	})
}

function reorderPadLayout(pads) {
	return [12, 13, 14, 15, 8, 9, 10, 11, 4, 5, 6, 7, 0, 1, 2, 3].map((index) => pads[index])
}

function linearOrganiseSamples(samples) {
	samples = samples.sort((a, b) => a.metadata.originalPath - b.metadata.originalPath)
	const pads = Array(16).fill(null)
	for (let i = 0; i < pads.length; i++) {
		pads[i] = samples[i] || null
	}
	const result = reorderPadLayout(pads.map((sample, index) => ({sample}))).map((entry) => ({
		sample: entry?.sample?.metadata?.originalPath || null,
	}))
	return result
}

function organiseSamples(samples, filters) {
	console.dir(samples)
	let samplesToOrganise = samples
	if (filters.length > 0) {
		console.log(filters)
		samplesToOrganise = samples.filter((sample) => {
			console.log(`Looking for ${sample.metadata.originalPath}`)
			return filters.includes(sample.metadata.originalPath)
		})
		console.dir(samplesToOrganise)
	}

	if (samplesToOrganise.length !== 16) {
		console.warn(
			`Samples doesnt have required length (its length is ${samplesToOrganise.length}). We are going to linear align, rather than smart assign`
		)
		console.dir(samplesToOrganise)
		return linearOrganiseSamples(samplesToOrganise)
	}

	let categorizedSamples = {}
	samplesToOrganise.forEach((sample) => {
		const firstTag = sample.metadata.tags[0]

		if (firstTag) {
			if (!categorizedSamples[firstTag]) {
				categorizedSamples[firstTag] = []
			}
			categorizedSamples[firstTag].push(sample.metadata.originalPath)
		}
	})

	for (let key in categorizedSamples) {
		categorizedSamples[key].sort()
	}

	const originalCatgorizedSamples = categorizedSamples
	const pads = Array(16).fill(null)
	const usedSamples = new Set()
	const row3Categories = ['Percussion', 'Kick', 'Snare', 'Hihat', 'Cymbal', 'FX']

	assignPads([0], ['Kick'], pads, usedSamples, categorizedSamples)
	assignPads([1], ['Snare'], pads, usedSamples, categorizedSamples)
	assignPads([2], ['Hihat'], pads, usedSamples, categorizedSamples)
	assignPads([3], ['Hihat', 'Clap'], pads, usedSamples, categorizedSamples)
	assignPads([4], ['Kick', 'Percussion'], pads, usedSamples, categorizedSamples)
	assignPads([5], ['Snare', 'Percussion'], pads, usedSamples, categorizedSamples)
	assignPads([6], ['Hihat', 'Percussion'], pads, usedSamples, categorizedSamples)
	assignPads([7], ['Hihat', 'Clap', 'Percussion'], pads, usedSamples, categorizedSamples)
	assignPads([8], row3Categories, pads, usedSamples, categorizedSamples)
	assignPads([9], row3Categories, pads, usedSamples, categorizedSamples)
	assignPads([10], row3Categories, pads, usedSamples, categorizedSamples)
	assignPads([11], row3Categories, pads, usedSamples, categorizedSamples)
	assignPads([12, 13, 14, 15], ['Bass, Synth, Loop'], pads, usedSamples, categorizedSamples)

	const unusedSamples = Object.values(categorizedSamples).flat()
	console.log('Assigning unused:')
	console.log(unusedSamples)
	pads.forEach((pad, index) => {
		if (!pad && unusedSamples.length > 0) {
			const sample = unusedSamples.shift()
			pads[index] = sample
			usedSamples.add(sample)
		}
	})

	pads.forEach((pad, index) => {
		if (!pad) {
			throw new Error(
				`Pad ${index} was not assigned a sample. Ensure you have enough samples.`
			)
		}
	})

	return reorderPadLayout(pads.map((sample, index) => ({sample})))
}
